@@ -1709,25 +1709,20 @@ static void WI_loadUnloadData(load_callback_t callback)
     char name[9];
     anim_t *a;
 
-    if (nervewadfile && gamemission == pack_nerve)
-    {
-	for (i=0 ; i<9 ; i++)
-	{
-	    DEH_snprintf(name, 9, "NWILV%2.2d", i);
-            callback(name, &lnames[i]);
-	}
-	for ( ; i<NUMCMAPS ; i++)
-	{
-	    DEH_snprintf(name, 9, "CWILV%2.2d", i);
-            callback(name, &lnames[i]);
-	}
-    }
-    else
     if (gamemode == commercial)
     {
 	for (i=0 ; i<NUMCMAPS ; i++)
 	{
 	    DEH_snprintf(name, 9, "CWILV%2.2d", i);
+	    // [crispy] NRFTL / The Master Levels
+	    if (crispy->havenerve && wbs->epsd == 1 && i < 9) // [crispy] gamemission == pack_nerve
+	    {
+		name[0] = 'N';
+	    }
+	    if (crispy->havemaster && crispy->havemaster != (char *)-1 && wbs->epsd == 2 && i < 21) // [crispy] gamemission == pack_master
+	    {
+		name[0] = 'M';
+	    }
             callback(name, &lnames[i]);
 	}
     }
