@@ -72,6 +72,11 @@ static int hours;
 static int minutes;
 static int seconds;
 
+// [crispy] Show total time on intermission
+static int totalHours;
+static int totalMinutes;
+static int totalSeconds;
+
 static int slaughterboy;        // in DM, the player with the most kills
 
 static int killPercent[MAXPLAYERS];
@@ -141,6 +146,17 @@ static yahpt_t YAHspot[3][9] = {
      }
 };
 
+static const char *NameForMap(int map)
+{
+    const char *name = LevelNames[(gameepisode - 1) * 9 + map - 1];
+    name = DEH_String(name);
+    if (strlen(name) < 7)
+    {
+        return "";
+    }
+    return name + 7;
+}
+
 //========================================================================
 //
 // IN_Start
@@ -218,6 +234,14 @@ void IN_InitStats(void)
         minutes = count / 60;
         count -= minutes * 60;
         seconds = count;
+
+        // [crispy] Show total time on intermission
+        count = totalleveltimes / 35;
+        totalHours = count / 3600;
+        count -= totalHours * 3600;
+        totalMinutes = count / 60;
+        count -= totalMinutes * 60;
+        totalSeconds = count;
     }
     else if (netgame && !deathmatch)
     {
@@ -596,12 +620,12 @@ void IN_DrawStatBack(void)
 
 void IN_DrawOldLevel(void)
 {
+    const char *level_name = NameForMap(prevmap);
     int i;
     int x;
 
-    x = 160 - MN_TextBWidth(LevelNames[(gameepisode - 1) * 9 + prevmap - 1] +
-                            7) / 2;
-    IN_DrTextB(LevelNames[(gameepisode - 1) * 9 + prevmap - 1] + 7, x, 3);
+    x = 160 - MN_TextBWidth(level_name) / 2;
+    IN_DrTextB(level_name, x, 3);
     x = 160 - MN_TextAWidth(DEH_String("FINISHED")) / 2;
     MN_DrTextA(DEH_String("FINISHED"), x, 25);
 
@@ -647,14 +671,14 @@ void IN_DrawOldLevel(void)
 
 void IN_DrawYAH(void)
 {
+    const char *level_name = NameForMap(gamemap);
     int i;
     int x;
 
     x = 160 - MN_TextAWidth(DEH_String("NOW ENTERING:")) / 2;
     MN_DrTextA(DEH_String("NOW ENTERING:"), x, 10);
-    x = 160 - MN_TextBWidth(LevelNames[(gameepisode - 1) * 9 + gamemap - 1] +
-                            7) / 2;
-    IN_DrTextB(LevelNames[(gameepisode - 1) * 9 + gamemap - 1] + 7, x, 20);
+    x = 160 - MN_TextBWidth(level_name) / 2;
+    IN_DrTextB(level_name, x, 20);
 
     if (prevmap == 9)
     {
@@ -685,16 +709,24 @@ void IN_DrawYAH(void)
 
 void IN_DrawSingleStats(void)
 {
+    const char *prev_level_name = NameForMap(prevmap);
+    const char *next_level_name = NameForMap(gamemap);
     int x;
     static int sounds;
 
-    IN_DrTextB(DEH_String("KILLS"), 50, 65);
-    IN_DrTextB(DEH_String("ITEMS"), 50, 90);
-    IN_DrTextB(DEH_String("SECRETS"), 50, 115);
+    // [crispy] offset the stats for Ep.4 and up, to make room for level time
+    int yoffset = 0;
+    if (gamemode == retail && gameepisode > 3)
+    {
+        yoffset = 20;
+    }
 
-    x = 160 - MN_TextBWidth(LevelNames[(gameepisode - 1) * 9 + prevmap - 1] +
-                            7) / 2;
-    IN_DrTextB(LevelNames[(gameepisode - 1) * 9 + prevmap - 1] + 7, x, 3);
+    IN_DrTextB(DEH_String("KILLS"), 50, 65 - yoffset);
+    IN_DrTextB(DEH_String("ITEMS"), 50, 90 - yoffset);
+    IN_DrTextB(DEH_String("SECRETS"), 50, 115 - yoffset);
+
+    x = 160 - MN_TextBWidth(prev_level_name) / 2;
+    IN_DrTextB(prev_level_name, x, 3);
     x = 160 - MN_TextAWidth(DEH_String("FINISHED")) / 2;
     MN_DrTextA(DEH_String("FINISHED"), x, 25);
 
@@ -708,9 +740,9 @@ void IN_DrawSingleStats(void)
         S_StartSound(NULL, sfx_dorcls);
         sounds++;
     }
-    IN_DrawNumber(players[consoleplayer].killcount, 200, 65, 3);
-    V_DrawShadowedPatch(237, 65, FontBSlash);
-    IN_DrawNumber(totalkills, 248, 65, 3);
+    IN_DrawNumber(players[consoleplayer].killcount, 200, 65 - yoffset, 3);
+    V_DrawShadowedPatch(237, 65 - yoffset, FontBSlash);
+    IN_DrawNumber(totalkills, 248, 65 - yoffset, 3);
     if (intertime < 60)
     {
         return;
@@ -720,9 +752,9 @@ void IN_DrawSingleStats(void)
         S_StartSound(NULL, sfx_dorcls);
         sounds++;
     }
-    IN_DrawNumber(players[consoleplayer].itemcount, 200, 90, 3);
-    V_DrawShadowedPatch(237, 90, FontBSlash);
-    IN_DrawNumber(totalitems, 248, 90, 3);
+    IN_DrawNumber(players[consoleplayer].itemcount, 200, 90 - yoffset, 3);
+    V_DrawShadowedPatch(237, 90 - yoffset, FontBSlash);
+    IN_DrawNumber(totalitems, 248, 90 - yoffset, 3);
     if (intertime < 90)
     {
         return;
@@ -732,9 +764,9 @@ void IN_DrawSingleStats(void)
         S_StartSound(NULL, sfx_dorcls);
         sounds++;
     }
-    IN_DrawNumber(players[consoleplayer].secretcount, 200, 115, 3);
-    V_DrawShadowedPatch(237, 115, FontBSlash);
-    IN_DrawNumber(totalsecret, 248, 115, 3);
+    IN_DrawNumber(players[consoleplayer].secretcount, 200, 115 - yoffset, 3);
+    V_DrawShadowedPatch(237, 115 - yoffset, FontBSlash);
+    IN_DrawNumber(totalsecret, 248, 115 - yoffset, 3);
     if (intertime < 150)
     {
         return;
@@ -747,18 +779,27 @@ void IN_DrawSingleStats(void)
 
     if (gamemode != retail || gameepisode <= 3)
     {
-        IN_DrTextB(DEH_String("TIME"), 85, 160);
-        IN_DrawTime(155, 160, hours, minutes, seconds);
+        IN_DrTextB(DEH_String("TIME"), 85, 150);
+        IN_DrawTime(155, 150, hours, minutes, seconds);
+
+        // [crispy] Show total time on intermission
+        IN_DrTextB(DEH_String("TOTAL"), 85, 170);
+        IN_DrawTime(155, 170, totalHours, totalMinutes, totalSeconds);
     }
     else
     {
+        // [crispy] show the level time for Ep.4 and up
+        IN_DrTextB(DEH_String("TIME"), 85, 120);
+        IN_DrawTime(155, 120, hours, minutes, seconds);
+
+        // [crispy] Show total time on intermission
+        IN_DrTextB(DEH_String("TOTAL"), 85, 140);
+        IN_DrawTime(155, 140, totalHours, totalMinutes, totalSeconds);
+
         x = 160 - MN_TextAWidth(DEH_String("NOW ENTERING:")) / 2;
         MN_DrTextA(DEH_String("NOW ENTERING:"), x, 160);
-        x = 160 -
-            MN_TextBWidth(LevelNames[(gameepisode - 1) * 9 + gamemap - 1] +
-                          7) / 2;
-        IN_DrTextB(LevelNames[(gameepisode - 1) * 9 + gamemap - 1] + 7, x,
-                   170);
+        x = 160 - MN_TextBWidth(next_level_name) / 2;
+        IN_DrTextB(next_level_name, x, 170);
         skipintermission = false;
     }
 }
@@ -771,6 +812,7 @@ void IN_DrawSingleStats(void)
 
 void IN_DrawCoopStats(void)
 {
+    const char *level_name = NameForMap(prevmap);
     int i;
     int x;
     int ypos;
@@ -780,9 +822,8 @@ void IN_DrawCoopStats(void)
     IN_DrTextB(DEH_String("KILLS"), 95, 35);
     IN_DrTextB(DEH_String("BONUS"), 155, 35);
     IN_DrTextB(DEH_String("SECRET"), 232, 35);
-    x = 160 - MN_TextBWidth(LevelNames[(gameepisode - 1) * 9 + prevmap - 1] +
-                            7) / 2;
-    IN_DrTextB(LevelNames[(gameepisode - 1) * 9 + prevmap - 1] + 7, x, 3);
+    x = 160 - MN_TextBWidth(level_name) / 2;
+    IN_DrTextB(level_name, x, 3);
     x = 160 - MN_TextAWidth(DEH_String("FINISHED")) / 2;
     MN_DrTextA(DEH_String("FINISHED"), x, 25);
 
@@ -925,6 +966,7 @@ void IN_DrawDMStats(void)
 //
 //========================================================================
 
+// [crispy] always draw seconds; don't 0-pad minutes with no hour
 void IN_DrawTime(int x, int y, int h, int m, int s)
 {
     if (h)
@@ -933,12 +975,15 @@ void IN_DrawTime(int x, int y, int h, int m, int s)
         IN_DrTextB(DEH_String(":"), x + 26, y);
     }
     x += 34;
-    if (m || h)
+    if (h || m > 9)
     {
         IN_DrawNumber(m, x, y, 2);
     }
+    else if (m)
+    {
+        IN_DrawNumber(m, x + 12, y, 1);
+    }
     x += 34;
-    if (s)
     {
         IN_DrTextB(DEH_String(":"), x - 8, y);
         IN_DrawNumber(s, x, y, 2);
