@@ -477,7 +477,7 @@ void R_DrawTopBorder(void)
     }
     dest = I_VideoBuffer;
 
-    for (y = 0; y < 30; y++)
+    for (y = 0; y < (30 << crispy->hires); y++)
     {
         for (x = 0; x < SCREENWIDTH / 64; x++)
         {
