@@ -56,23 +56,23 @@ short openings[MAXOPENINGS], *lastopening;
 // Clip values are the solid pixel bounding the range.
 // floorclip start out SCREENHEIGHT
 // ceilingclip starts out -1
-short floorclip[SCREENWIDTH];
-short ceilingclip[SCREENWIDTH];
+short floorclip[MAXWIDTH];
+short ceilingclip[MAXWIDTH];
 
 // spanstart holds the start of a plane span, initialized to 0
-int spanstart[SCREENHEIGHT];
-int spanstop[SCREENHEIGHT];
+int spanstart[MAXHEIGHT];
+int spanstop[MAXHEIGHT];
 
 // Texture mapping
 lighttable_t **planezlight;
 fixed_t planeheight;
-fixed_t yslope[SCREENHEIGHT];
-fixed_t distscale[SCREENWIDTH];
+fixed_t yslope[MAXHEIGHT];
+fixed_t distscale[MAXWIDTH];
 fixed_t basexscale, baseyscale;
-fixed_t cachedheight[SCREENHEIGHT];
-fixed_t cacheddistance[SCREENHEIGHT];
-fixed_t cachedxstep[SCREENHEIGHT];
-fixed_t cachedystep[SCREENHEIGHT];
+fixed_t cachedheight[MAXHEIGHT];
+fixed_t cacheddistance[MAXHEIGHT];
+fixed_t cachedxstep[MAXHEIGHT];
+fixed_t cachedystep[MAXHEIGHT];
 
 // PRIVATE DATA DEFINITIONS ------------------------------------------------
 
@@ -384,7 +384,7 @@ void R_DrawPlanes(void)
     int skyTexture2;
     int scrollOffset;
     int frac;
-    int fracstep = FRACUNIT >> hires;
+    int fracstep = FRACUNIT >> crispy->hires;
 
     extern byte *ylookup[MAXHEIGHT];
     extern int columnofs[MAXWIDTH];
