@@ -214,6 +214,7 @@ void V_DrawPatch(int x, int y, patch_t *patch)
 
     y -= SHORT(patch->topoffset);
     x -= SHORT(patch->leftoffset);
+    x += WIDESCREENDELTA; // [crispy] horizontal widescreen offset
 
     // haleyjd 08/28/10: Strife needs silent error checking here.
     if(patchclip_callback)
@@ -307,14 +308,20 @@ void V_DrawPatchFullScreen(patch_t *patch, boolean flipped)
     const short width = SHORT(patch->width);
     const short height = SHORT(patch->height);
 
-    dx = (SCREENWIDTH << FRACBITS) / width;
-    dxi = (width << FRACBITS) / SCREENWIDTH;
+    dx = (NONWIDEWIDTH << FRACBITS) / width;
+    dxi = (width << FRACBITS) / NONWIDEWIDTH;
     dy = (SCREENHEIGHT << FRACBITS) / height;
     dyi = (height << FRACBITS) / SCREENHEIGHT;
 
     patch->leftoffset = 0;
     patch->topoffset = 0;
 
+    // [crispy] fill pillarboxes in widescreen mode
+    if (SCREENWIDTH != NONWIDEWIDTH)
+    {
+        V_DrawFilledBox(0, 0, SCREENWIDTH, SCREENHEIGHT, 0);
+    }
+
     if (flipped)
     {
         V_DrawPatchFlipped(0, 0, patch);
@@ -324,8 +331,8 @@ void V_DrawPatchFullScreen(patch_t *patch, boolean flipped)
         V_DrawPatch(0, 0, patch);
     }
 
-    dx = (SCREENWIDTH << FRACBITS) / ORIGWIDTH;
-    dxi = (ORIGWIDTH << FRACBITS) / SCREENWIDTH;
+    dx = (NONWIDEWIDTH << FRACBITS) / ORIGWIDTH;
+    dxi = (ORIGWIDTH << FRACBITS) / NONWIDEWIDTH;
     dy = (SCREENHEIGHT << FRACBITS) / ORIGHEIGHT;
     dyi = (ORIGHEIGHT << FRACBITS) / SCREENHEIGHT;
 }
@@ -348,6 +355,7 @@ void V_DrawPatchFlipped(int x, int y, patch_t *patch)
  
     y -= SHORT(patch->topoffset); 
     x -= SHORT(patch->leftoffset); 
+    x += WIDESCREENDELTA; // [crispy] horizontal widescreen offset
 
     // haleyjd 08/28/10: Strife needs silent error checking here.
     if(patchclip_callback)
@@ -845,10 +853,10 @@ void V_DrawRawScreen(pixel_t *raw)
 void V_Init (void) 
 { 
     // [crispy] initialize resolution-agnostic patch drawing
-    if (SCREENWIDTH && SCREENHEIGHT)
+    if (NONWIDEWIDTH && SCREENHEIGHT)
     {
-        dx = (SCREENWIDTH << FRACBITS) / ORIGWIDTH;
-        dxi = (ORIGWIDTH << FRACBITS) / SCREENWIDTH;
+        dx = (NONWIDEWIDTH << FRACBITS) / ORIGWIDTH;
+        dxi = (ORIGWIDTH << FRACBITS) / NONWIDEWIDTH;
         dy = (SCREENHEIGHT << FRACBITS) / ORIGHEIGHT;
         dyi = (ORIGHEIGHT << FRACBITS) / SCREENHEIGHT;
     }
